/* 
-----------------------------------------------
Generic Allegro Project Template
By Kronoman - July 2003
Copyright (c) 2003, Kronoman
In loving memory of my father
-----------------------------------------------
gerror.h
-----------------------------------------------
Error messages
----------------------------------------------- 
*/
#ifndef GERROR_H
#define GERROR_H

#include <allegro.h>
#include <stdarg.h> /* for the variable argument list */
#include <stdlib.h> /* for use of malloc */

void raise_error(AL_CONST char *msg, ...);

#endif
